# stores data on environment
# initialized when initial data is being sent through EnvChannel
import json

from Node import Node
from ta_state import Location
import tp_task


class EnvironmentData:

    def __init__(self):
        self.matrix = None
        self.rotation = None
        self.action_size = None
        self.observation_size = None
        self.number_of_robots = None
        self.number_of_current_tasks = None
        self.can_go_backwards = False
        self.random_tasks = None  # if set to true random tasks will be send whenever a task has been completed,
        # otherwise all tasks self.tasks_to_be_assigned

        self.instance_ids = []
        self.current_nodes_dictionary = {} # dictionary instance_id -> current node
        self.tasks_to_be_assigned = [] # tasks that are still to be assigned
        self.assigned_tasks_dictionary = {} # dictionary instance_id -> assigned task

        self.task_side_channel = None

        self.tasksCreated = 0

    #########################################
    ## methods used by EnvChannel to initialize attributes:
    #########################################

    # set_initial_nodes_dictionary
    # set_tasks_to_be_assigned
    # set_assigned_tasks

    # given a correct string containing array of nodes
    def set_initial_nodes_dictionary(self, initial_nodes):
        for i in range(self.number_of_robots):
            self.current_nodes_dictionary[self.instance_ids[i]] = Node(initial_nodes[i]["GridPosition"]["Row"],
                                                      initial_nodes[i]["GridPosition"]["Column"],
                                                      initial_nodes[i]["Degrees"])

    # given a correct json_string containing array of tasks
    def set_tasks_to_be_assigned(self, tasks):

        for i in range(self.number_of_current_tasks):
            pickup = Location(tasks[i]["PickUp"]["Column"], tasks[i]["PickUp"]["Row"], 0)
            delivery = Location(tasks[i]["Delivery"]["Column"], tasks[i]["Delivery"]["Row"], 0)
            releaseTime = tasks[i]["ReleaseTime"]
            id = tasks[i]["TaskId"]
            task = tp_task.Task(id, pickup, delivery, releaseTime=releaseTime)
            self.tasks_to_be_assigned.append(task)

            #tp_task.CURRENT_TASK_ID += 1

    # initialize dictionary robot instance_id -> task (None)
    def set_assigned_tasks(self):
        for i in range(self.number_of_robots):
            self.assigned_tasks_dictionary[self.instance_ids[i]] = None

    #########################################
    ## methods used to update environment info
    #########################################

    # when a task is assigned to a robot, it has to be added to self.assigned_tasks
    # and removed from self.tasks_to_be_assigned
    # info has to be sent to unity via the correct channel
    def assign_task(self, robot_instanceID, task):
        self.assigned_tasks_dictionary[robot_instanceID] = task
        # self.tasks_to_be_assigned.remove(task)
        taskDict = {"TaskID" : str(task.id),
                    "InstanceID": str(robot_instanceID)}
        self.task_side_channel.send_string(json.dumps(taskDict))

    # when a new task is available it is added to tasks_to_be_assigned
    # a check is made whether there are still tasks that can be added, but this should not be happening (diagnostical)
    def add_task_to_tasks_to_be_assigned(self, task):
        if len(self.tasks_to_be_assigned) == self.number_of_current_tasks:
            raise Exception("cannot add more tasks")
        self.tasks_to_be_assigned.append(task)

    # when a robot has finished his task it has to be removed from assigned_tasks
    def task_done(self, instance_id):
        self.assigned_tasks_dictionary[instance_id] = None
