# Token Passing (TP)

The Token-Passing (TP) algorithm is a decentralized algorithm for the online variant of the MAPD problem.

In the online variant of the MAPD problem the following is supported:
- Using randomly generated tasks (randomtasks = True)
- Using tasks from a taskslist (bv. ../config/tasks.txt)
    -> When using a taskslist with releaseTimes: releaseTimes are ignored (only used in offline variant)
- Tested from 1 agent up untill 50 agents (significant lag for larger amount of agents)

## Running the algorithm

1. Install the virtual environment
    - pipenv install
2. Run the "decentralized_decisionmaker_shortest_path.py" file
    - pipenv run python decentralized_decisionmaker_shortest_path.py
3. Start the game inside the Unity editor

## Working of the algorithm

This implementation is based on the "Lifelong Multi-Agent Path Finding for Online Pickup and Delivery Tasks" paper (https://arxiv.org/pdf/1705.10868.pdf).
An explanation of the algorithm is available at:
    - Section 4.1 of the "Lifelong Multi-Agent Path Finding for Online Pickup and Delivery Tasks" paper
    - Section 2.2.1 of the paper provided in the "paper" directory

General overview of working:
    - Token contains:
        * Paths of all agents
        * Taskset
        * Assignments of tasks of all agents
    - In each timestep each agent moves one step following its path in the token
    - When an agent reaches the end of its path (= free agent) it looks for a task:
        - Task found: 
            * Plan path to execute task and update token
            * Remove the selected task from the task set
        - No task found: 
            * Plan path to safe location and update token

## Overview of the provided files

Main file:
- decentralized_decisionmaker_shortest_path.py: Main file that runs the algorithm

Token Passing:
- tp_agent.py: Agent that are executing tasks
- tp_system: System manages the agents and tasks
- tp_task: Pickup and delivery tasks
- tp_token: Token that is used for communicating global information between the agents

A-star:
- ta_astar.py: A-Star algorithm
- ta_state.py: Contains Location and state classes used for the path planning