CURRENT_TASK_ID = 0

class Task:
    """ Represents a task in the pickup & delivery problem 

        Attributes:
        id: int 
            Identifier of the task
        pickupLocation: Position 
            The location where the task needs to be picked up
        deliveryLocation: Position 
            The location where the task needs to be delivered
    """
    def __init__(self, id, pickupLocation, deliveryLocation, releaseTime=0):
        self.id = id

        self.pickup = pickupLocation
        self.delivery = deliveryLocation

        self.releaseTime = releaseTime
        
        # Keep track timestep when task enters the taskset and when it's executed => calculating service time
        self.entered = None
        self.delivered = None

    def service_time(self):
        if self.entered is not None and self.delivered is not None:
            return self.delivered - self.entered
        else:
            return None

    def __eq__(self, other):
        return self.pickup == other.pickup and self.delivery == other.delivery

    def __hash__(self):
        return ((hash(self.pickup) * 397) ** hash(self.delivery)) * 397
        
    # Better information for debugging
    def __repr__(self):
        return f"Task({self.id},Pickup({self.pickup}), Delivery({self.delivery}))"

    def __str__(self):
        return f"Task({self.id},Pickup({self.pickup}), Delivery({self.delivery}))"
