class Token:
    def __init__(self):
        # Keep track of the paths of all the agents
        self.paths = dict()
        # Keep track of which agent is executing which task
        self.assignments = dict()
        # Keep track of the remaining tasks
        self.taskSet = []

    ##---------##
    ## Getters ##
    ##---------##

    # Returns the last location on the path of each agent
    def get_pathends(self, given_id=None):
        pathEnds = []
        for a_id, k in self.paths.items():
            if given_id is None or a_id != given_id:
                pathEnds.append(k[list(k)[-1]])
        return pathEnds

    # Get the paths of all agents except the path of agent with ID = own_id
    def get_other_agents_paths(self, timestep, own_id):
        other_agent_paths = []
        for a_id, path in self.paths.items():
            if a_id != own_id:
                other_agent_paths.append(path)
        return other_agent_paths

    ##------------------##
    ## Add new elements ##
    ##------------------##

    # Adds a new agent to the system
    def add_agent(self, agent):
        a_id = agent.id
        self.paths[a_id] = dict()
        self.paths[a_id][0] = agent.pos
        self.assignments[a_id] = None
        
    # Adds a new task to the system
    def add_task(self, task):
        self.taskSet.append(task)
        
    ##-------------------##
    ## Utilitary methods ##
    ##-------------------##
        
    # Restores the token with an old state of the token        
    def restore(self, old_token):
        self.paths = old_token.paths
        self.assignments = old_token.assignments
        self.taskSet = old_token.taskSet
        
    # Generates a copy of this token
    def copy(self):
        new_token = Token()
        new_token.paths = self.paths
        new_token.assignments = self.assignments
        new_token.taskSet = self.taskSet
        return new_token

    
